@@ -38,6 +38,7 @@ what could be done.
    The sPickle package is currently requires Stackless Python 2.7.
 
 .. autoclass:: Pickler
+   :members: analysePicklerStack
 
 .. autoclass:: SPickleTools
    :members:
@@ -56,7 +57,8 @@ del Pickler
 from ._sPickle import ( Pickler,
                         SPickleTools,
                         MODULE_TO_BE_PICKLED_FLAG_NAME,
-                        StacklessTaskletReturnValueException ) 
+                        StacklessTaskletReturnValueException,
+                        RESOURCE_TYPES )
 from pickle import __all__
 
 __all__ = __all__[:]
