@@ -22,6 +22,7 @@ from unittest import TestCase, skipIf
 
 import pickletools
 from StringIO import StringIO
+import cStringIO
 import pickle
 import types
 import sys
@@ -31,11 +32,13 @@ import thread
 import functools
 # a pure python module used for testing
 import tabnanny
-import os
+import os.path
 import socket
 import collections
 import logging
 import operator
+import weakref
+
 try:
     import gtk
 except ImportError:
@@ -114,6 +117,9 @@ class PlainClass(object):
     def isOk(self):
         return self.state == "OK"
     
+class PlainSubClass(PlainClass):
+    pass
+    
 class PlainClassicClass:
     def __init__(self, state = None):
         self.state = state 
@@ -250,7 +256,12 @@ class TestImportFunctor(object):
         
         for m in self.modulesToUnimport:
             if isinstance(m, types.ModuleType):
-                m = m.__name__
+                name = m.__name__
+                if m is not sys.modules.get(name):
+                    for k,v in sys.modules.iteritems():
+                        if v is m:
+                            name = k
+                m = name
             if sys.modules.has_key(m):
                 self.unimported.add(sys.modules[m])
                 del sys.modules[m]
@@ -291,7 +302,7 @@ class ClassWithHostile__getattribute__(object):
         
         raise Exception("This __getattribute__ always raises an exception: %r" % name)
         
-class PickelingTest(TestCase):
+class PicklingTest(TestCase):
     """
     This is the description of the class WfBaseTest. 
     
@@ -321,7 +332,7 @@ class PickelingTest(TestCase):
         dis = kw.get("dis")
         try:
             toBeDumped = (preObjects, obj[0] if len(obj) == 1 else obj )
-            p = self.pickler.dumps(toBeDumped)
+            p = self.pickler.dumps(toBeDumped, mangleModuleName=kw.get("mangleModuleName"))
             self.pickler.dis(p, out=StringIO())
         except:
             exinfo = sys.exc_info()
@@ -453,6 +464,141 @@ class PickelingTest(TestCase):
         obj, tif = self.wfModuleTest(anonymousWfModule, anonymousWfModule.__dict__)
         self.assertFalse(obj.__name__ in tif.post_modules)
 
+    def testModuleWithWrongName(self):
+        from . import mod_with_wrong_name
+        orig = mod_with_wrong_name
+        self.assertNotIn(orig.__name__, sys.modules)
+        self.assertIn(orig.NAME, sys.modules)
+        obj, tif = self.moduleTest(orig)
+        self.assertNotIn(orig.__name__, tif.post_modules)
+        self.assertIs(obj, tif.post_modules[orig.NAME])
+    def testModuleWithWrongNameU(self):
+        from . import mod_with_wrong_name
+        orig = mod_with_wrong_name
+        self.assertNotIn(orig.__name__, sys.modules)
+        self.assertIn(orig.NAME, sys.modules)
+        obj, tif = self.moduleTest(orig, unimport=True)
+        self.assertNotIn(orig.__name__, tif.post_modules)
+        self.assertIs(obj, tif.post_modules[orig.NAME])
+
+    class MangleModuleName(object):
+        """Add a prefix to certain module names"""
+        def __init__(self, start, prefix, package=None):
+            """
+            Create a MangleModuleName
+            
+            :param start: mangle modules starting with *start*
+            :param prefix: add *prefix* the name of a mangled module
+            
+            """
+            self.start = start
+            self.prefix = prefix
+            self.package = package
+        def __call__(self, pickler, name, module):
+            """
+            A sPickle.Pickler mangleModuleName functor
+            """
+            if module is os.path:
+                return "os.path"
+            if isinstance(name, str) and (name.startswith(self.start) or name == self.package):
+                prefix = self.prefix
+                class ReplacedModuleName(object):
+                    def __reduce__(self):
+                        return (operator.add, (prefix, name))               
+                return ReplacedModuleName()
+            return name
+        def getMangledName(self, name, module=None):
+            """Unit test helper function"""
+            if module is os.path:
+                return "os.path"
+            if isinstance(name, str) and (name.startswith(self.start) or name == self.package):
+                return self.prefix + name
+            return name
+
+    def testModule_MangleModuleName(self):
+        orig = tabnanny
+        self.assertTrue(orig.__name__ in sys.modules)
+        mmn = self.MangleModuleName(orig.__name__, "renamed_", package=orig.__name__)
+        mangledName = mmn.getMangledName(orig.__name__)
+
+        replacedObj = types.ModuleType(mangledName)
+        replacedObj.__dict__.update(orig.__dict__)
+        replacedObj.__dict__['__name__'] = mangledName
+        sys.modules[mangledName] = replacedObj
+        try:
+            obj, tif = self._moduleTestCommon(orig, mangleModuleName=mmn)
+        finally:
+            del sys.modules[mangledName]
+
+        self.assertTrue(obj is tif.post_modules[mangledName])
+
+    def testWfModule_MangleModuleName(self):
+        orig = wf_module
+        self.assertTrue(orig.__name__ in sys.modules)
+        self.assertIsInstance(orig.__package__, str)
+        package = orig.__name__.rpartition('.')[0]
+        mmn = self.MangleModuleName(package + ".wf_mod", "renamed_", package=package)
+        self.assertFalse(mmn.getMangledName(orig.__name__) in sys.modules)
+
+        obj, tif = self.wfModuleTest(orig, mangleModuleName=mmn, dis=False)
+
+        self.assertTrue(obj is tif.post_modules[mmn.getMangledName(orig.__name__)])
+        self.assertIsInstance(obj.__package__, str)
+        self.assertEqual(obj.__package__, mmn.getMangledName(orig.__package__))
+
+    def testWfModuleU_MangleModuleName(self):
+        orig = wf_module
+        self.assertTrue(orig.__name__ in sys.modules)
+        mmn = self.MangleModuleName(orig.__name__, "renamed_")
+        self.assertFalse(mmn.getMangledName(orig.__name__) in sys.modules)
+        obj, tif = self.wfModuleTest(orig, mangleModuleName=mmn, unimport=True)
+        self.assertEqual(obj, tif.post_modules.get(mmn.getMangledName(orig.__name__)))
+        self.assertTrue(obj is tif.post_modules[mmn.getMangledName(orig.__name__)])
+
+    def testAnonymousWfModule_MangleModuleName(self):
+        self.assertFalse(anonymousWfModule.__name__ in sys.modules)
+        mmn = self.MangleModuleName(anonymousWfModule.__name__, "renamed_")
+        obj, tif = self.wfModuleTest(anonymousWfModule, mangleModuleName=mmn, preObjects=anonymousWfModule.__name__, dis=False)
+        self.assertFalse(obj.__name__ in tif.post_modules)
+
+    def testAnonymousWfModule2_MangleModuleName(self):
+        self.assertFalse(anonymousWfModule.__name__ in sys.modules)
+        mmn = self.MangleModuleName(anonymousWfModule.__name__, "renamed_")
+        obj, tif = self.wfModuleTest(anonymousWfModule, mangleModuleName=mmn, preObjects=anonymousWfModule.__dict__, dis=False)
+        self.assertFalse(obj.__name__ in tif.post_modules)
+
+    def testOsPath_MangleModuleName(self):
+        orig = os.path
+        self.assertNotEqual(orig.__name__, "os.path")
+        mmn = self.MangleModuleName("DOES NOT APPLY", "", package=orig.__name__)
+        p = self.dumpWithPreobjects(None, orig, dis=False, 
+                                    mangleModuleName=mmn
+                                    )
+
+        il = self.pickler.getImportList(p)
+        modules = set([i.partition(" ")[0] for i in il])
+        self.assertNotIn(orig.__name__, modules)
+        self.assertIn("os.path", modules)
+        
+        obj = self.pickler.loads(p)[1]
+        self.assertIs(obj, orig)
+
+    def testOsPathJoin_MangleModuleName(self):
+        orig = os.path
+        self.assertNotEqual(orig.__name__, "os.path")
+        mmn = self.MangleModuleName("DOES NOT APPLY", "", package=orig.__name__)
+        p = self.dumpWithPreobjects(None, orig.join, dis=False, 
+                                    mangleModuleName=mmn
+                                    )
+
+        il = self.pickler.getImportList(p)
+        modules = set([i.partition(" ")[0] for i in il])
+        self.assertNotIn(orig.__name__, modules)
+        self.assertIn("os.path", modules)
+        
+        obj = self.pickler.loads(p)[1]
+        self.assertIs(obj, orig.join)
+
     @skipIf(gtk is None, "gtk not available")
     def testGtk(self):
         # the pure stackless pickler fails to import gtk
@@ -475,28 +621,40 @@ class PickelingTest(TestCase):
         self.assertFalse(orig.__name__ in tif.names, "Import of forbidden module %r" % (orig.__name__,))
         return (obj, tif)
         
-    def _moduleTestCommon(self, module, preObjects=None, unimport=(), dis=False):
+    def _moduleTestCommon(self, module, preObjects=None, unimport=(), dis=False, mangleModuleName=None):
         orig = module
-        p = self.dumpWithPreobjects(preObjects, orig, dis=dis)
+        p = self.dumpWithPreobjects(preObjects, orig, dis=dis, mangleModuleName=mangleModuleName)
         
         if unimport:
             if unimport is True:
                 unimport = (orig,)
         
-        with TestImportFunctor(*unimport) as tif:    
-            obj = self.pickler.loads(p)[-1]
+        with TestImportFunctor(*unimport) as tif:
+            # cPickle does not use our modified __import__ function for the GLOBAL op code
+            obj = self.pickler.loads(p, useCPickle=False)[-1]
 
         # test obj        
         self.assertEquals(type(obj), type(orig))
         self.assertTrue(type(obj) is type(orig))
-        self.assertEquals(obj.__name__, orig.__name__)
+        oname = orig.__name__
+        if mangleModuleName:
+            oname = mangleModuleName.getMangledName(oname)
+        self.assertEquals(obj.__name__, oname)
         self.assertEqual(set(obj.__dict__.iterkeys()), set(orig.__dict__.iterkeys()))
         if callable(getattr(orig, "isOk", None)):
             self.assertTrue(obj.isOk() is True)
         
         if tif.pre_modules.has_key(orig.__name__):
-            # the import of the module must not change anything 
-            self.assertEqual(tif.pre_modules, tif.post_modules)
+            # the import of the module must not change anything
+            if mangleModuleName is None:
+                self.assertEqual(tif.pre_modules, tif.post_modules)
+            else:
+                post = dict(tif.post_modules)
+                for k in tif.pre_modules.keys():
+                    mangled = mangleModuleName.getMangledName(k)
+                    if mangled != k and k != mangleModuleName.package:
+                        del post[mangled]
+                self.assertEqual(tif.pre_modules, post)
             
         return (obj, tif)
 
@@ -583,9 +741,74 @@ class PickelingTest(TestCase):
         # Todo: compare the other attributes of a function
         return obj
 
+    # Tests for function creation
+    def testTypeCode(self):
+        p = self.pickler.dumps(types.CodeType)
+        obj = self.pickler.loads(p)
+        self.assertTrue(obj is types.CodeType)
+
+    def testTypeCell(self):
+        cellType = type((lambda: self).func_closure[0])
+        p = self.pickler.dumps(cellType)
+        obj = self.pickler.loads(p)
+        self.assertTrue(obj is cellType)
+    
+    def testCodeObject(self):
+        # a function code object
+        orig = self.testCodeObject.im_func.func_code
+        self.codeObjectTest(orig, dis=False)
+
+    def codeObjectTest(self, orig, dis=False):
+        # a function code object
+        self.assertIsInstance(orig, types.CodeType)
+        p = self.dumpWithPreobjects(None, orig, dis=dis)
+        obj = self.pickler.loads(p)[1]
+        self.assertIsNot(obj, orig)
+        self.assertIsInstance(obj, types.CodeType)
+        self.assertEqual(obj.co_name, orig.co_name)
+        self.assertEqual(obj.co_argcount, orig.co_argcount)
+        self.assertEqual(obj.co_nlocals, orig.co_nlocals)
+        self.assertEqual(obj.co_varnames, orig.co_varnames)
+        self.assertEqual(obj.co_cellvars, orig.co_cellvars)
+        self.assertEqual(obj.co_freevars, orig.co_freevars)
+        self.assertEqual(obj.co_code, orig.co_code)
+        self.assertEqual(obj.co_consts, orig.co_consts)
+        self.assertEqual(obj.co_names, orig.co_names)
+        self.assertEqual(obj.co_filename, orig.co_filename)
+        self.assertEqual(obj.co_firstlineno, orig.co_firstlineno)
+        self.assertEqual(obj.co_lnotab, orig.co_lnotab)
+        self.assertEqual(obj.co_stacksize, orig.co_stacksize)
+        self.assertEqual(obj.co_flags, orig.co_flags)
+        
 
     # Tests for special objects and types
+    def testTypeWrapperDescriptor(self):
+        p = self.pickler.dumps(_sPickle.WRAPPER_DESCRIPTOR_TYPE)
+        obj = self.pickler.loads(p)
+        self.assertIs(obj, _sPickle.WRAPPER_DESCRIPTOR_TYPE)
 
+    def testTypeMethodDescriptor(self):
+        p = self.pickler.dumps(_sPickle.METHOD_DESCRIPTOR_TYPE)
+        obj = self.pickler.loads(p)
+        self.assertIs(obj, _sPickle.METHOD_DESCRIPTOR_TYPE)
+
+    def testTypeMethodWrapper(self):
+        p = self.pickler.dumps(_sPickle.METHOD_WRAPPER_TYPE)
+        obj = self.pickler.loads(p)
+        self.assertIs(obj, _sPickle.METHOD_WRAPPER_TYPE)
+
+    def testDictSysModules(self):
+        p = self.pickler.dumps(sys.modules)
+        obj = self.pickler.loads(p)
+        self.assertTrue(obj is sys.modules)
+
+    def testDict__builtins__(self):
+        from __builtin__ import __dict__ as bid
+        self.assertIs(bid, __builtins__)
+        p = self.dumpWithPreobjects(None, __builtins__, dis=False)
+        obj = self.pickler.loads(p)[1]
+        self.assertIs(obj, __builtins__)
+        
     def testTypeType(self):
         p = self.pickler.dumps(type)
         obj = self.pickler.loads(p)
@@ -881,7 +1104,193 @@ class PickelingTest(TestCase):
         self.assertEqual(orig.__name__, obj.__name__)
         self.assertClassEquals(orig.__objclass__, obj.__objclass__)
 
+    def testCStringIoInputType(self):
+        orig = cStringIO.InputType
+        p = self.dumpWithPreobjects(None, orig)
+        obj = self.pickler.loads(p)[-1]
+        self.assertIs(obj, orig)
+
+    def testCStringIoOutputType(self):
+        orig = cStringIO.OutputType
+        p = self.dumpWithPreobjects(None, orig)
+        obj = self.pickler.loads(p)[-1]
+        self.assertIs(obj, orig)
+                
+    def testCStringIoOutput(self):
+        orig = cStringIO.StringIO()
+
+        p = self.dumpWithPreobjects(None, orig)
+        obj = self.pickler.loads(p)[-1]
+        self.assertIsNot(obj, orig)
+        self.assertIsInstance(obj, type(orig))
+        self.assertEqual(obj.getvalue(), orig.getvalue())
+        self.assertEqual(obj.tell(), orig.tell())
         
+        orig.write("0123456789")
+        
+        p = self.dumpWithPreobjects(None, orig)
+        obj = self.pickler.loads(p)[-1]
+        self.assertIsNot(obj, orig)
+        self.assertIsInstance(obj, type(orig))
+        self.assertEqual(obj.getvalue(), orig.getvalue())
+        self.assertEqual(obj.tell(), orig.tell())
+
+        orig.seek(5,0)
+
+        p = self.dumpWithPreobjects(None, orig)
+        obj = self.pickler.loads(p)[-1]
+        self.assertIsNot(obj, orig)
+        self.assertIsInstance(obj, type(orig))
+        self.assertEqual(obj.getvalue(), orig.getvalue())
+        self.assertEqual(obj.tell(), orig.tell())
+        
+        orig.close()
+        self.assertRaises(ValueError, orig.getvalue)
+        p = self.dumpWithPreobjects(None, orig)
+        obj = self.pickler.loads(p)[-1]
+        self.assertIsNot(obj, orig)
+        self.assertIsInstance(obj, type(orig))
+        self.assertRaises(ValueError, obj.getvalue)
+        
+    def testCStringIoInput(self):
+        orig = cStringIO.StringIO("")
+
+        p = self.dumpWithPreobjects(None, orig)
+        obj = self.pickler.loads(p)[-1]
+        self.assertIsNot(obj, orig)
+        self.assertIsInstance(obj, type(orig))
+        self.assertEqual(obj.getvalue(), orig.getvalue())
+        self.assertEqual(obj.tell(), orig.tell())
+        
+        orig = cStringIO.StringIO("0123456789")
+        
+        p = self.dumpWithPreobjects(None, orig)
+        obj = self.pickler.loads(p)[-1]
+        self.assertIsNot(obj, orig)
+        self.assertIsInstance(obj, type(orig))
+        self.assertEqual(obj.getvalue(), orig.getvalue())
+        self.assertEqual(obj.tell(), orig.tell())
+
+        orig.seek(5,0)
+
+        p = self.dumpWithPreobjects(None, orig)
+        obj = self.pickler.loads(p)[-1]
+        self.assertIsNot(obj, orig)
+        self.assertIsInstance(obj, type(orig))
+        self.assertEqual(obj.getvalue(), orig.getvalue())
+        self.assertEqual(obj.tell(), orig.tell())
+        
+        orig.close()
+        self.assertRaises(ValueError, orig.getvalue)
+        p = self.dumpWithPreobjects(None, orig)
+        obj = self.pickler.loads(p)[-1]
+        self.assertIsNot(obj, orig)
+        self.assertIsInstance(obj, type(orig))
+        self.assertRaises(ValueError, obj.getvalue)
+
+    def testOrderedDict(self):
+        # the method __reduce__ of collections.OrderedDict fails for recursive
+        # dictionaries. Test our fix
+        orig = collections.OrderedDict([['key1', 'value1'], ['key2', 'value2']])
+        orig['self'] = orig # a recursive dict. 
+        p = self.dumpWithPreobjects(None, orig)
+        obj = self.pickler.loads(p)[-1]
+        self.assertIsNot(obj, orig)
+        self.assertIsInstance(obj, collections.OrderedDict)
+        self.assertIs(obj, obj['self'])
+        # assertEqual fails to handle a recursive object
+        del obj['self']
+        del orig['self']
+        self.assertEqual(obj, orig)
+
+    def testTypeWeakrefReferenceType(self):
+        orig = weakref.ReferenceType
+        p = self.dumpWithPreobjects(None, orig)
+        obj = self.pickler.loads(p)[-1]
+        self.assertIs(obj, orig)
+
+    def testTypeWeakrefProxyType(self):
+        orig = weakref.ProxyType
+        p = self.dumpWithPreobjects(None, orig)
+        obj = self.pickler.loads(p)[-1]
+        self.assertIs(obj, orig)
+
+    def testTypeWeakrefCallableProxyType(self):
+        orig = weakref.CallableProxyType
+        p = self.dumpWithPreobjects(None, orig)
+        obj = self.pickler.loads(p)[-1]
+        self.assertIs(obj, orig)
+
+    def testWeakrefRef1(self):
+        something = PlainClass(123)
+        orig = weakref.ref(something)
+        p = self.dumpWithPreobjects(something, orig, dis=False)
+        obj, obj2 = self.pickler.loads(p)
+        self.assertIsNot(obj, something)
+        self.assertIsInstance(obj, type(something))
+        self.assertIsNot(obj2, orig)
+        self.assertIsInstance(obj2, type(orig))
+        self.assertIs(obj2(), obj)
+
+    def testWeakrefRef2(self):
+        # dead ref case
+        orig = weakref.ref(PlainClass(123))
+        p = self.dumpWithPreobjects(None, orig, dis=False)
+        obj = self.pickler.loads(p)[-1]
+        self.assertIsNot(obj, orig)
+        self.assertIsInstance(obj, type(orig))
+        self.assertIsNone(obj())
+
+    def testTypeSuper(self):
+        orig = super
+        self.assertIs(orig, type(super(PlainClass)))
+        self.assertIs(orig, type(super(PlainClass, PlainClass())))
+        p = self.dumpWithPreobjects(None, orig)
+        obj = self.pickler.loads(p)[-1]
+        self.assertIs(obj, orig)
+
+    def testSuperUnbound(self):
+        orig = super(PlainClass)
+        self.assertIsNone(orig.__self__)
+        self.assertIsNone(orig.__self_class__)
+        self.assertIs(orig.__thisclass__, PlainClass)
+        p = self.dumpWithPreobjects(None, orig, dis=False)
+        obj = self.pickler.loads(p)[-1]
+        self.assertIsNot(obj, orig)
+        self.assertIsInstance(obj, type(orig))
+        self.assertIs(obj.__thisclass__, orig.__thisclass__)
+        self.assertIsNone(obj.__self__)
+        self.assertIsNone(obj.__self_class__)
+
+    def testSuperBoundObj(self):
+        p = PlainSubClass()
+        orig = super(PlainClass, p)
+        self.assertIs(orig.__self__, p)
+        self.assertIs(orig.__thisclass__, PlainClass)
+        self.assertIs(orig.__self_class__, PlainSubClass)
+        p = self.dumpWithPreobjects(None, orig)
+        obj = self.pickler.loads(p)[-1]
+        self.assertIsNot(obj, orig)
+        self.assertIsInstance(obj, type(orig))
+        self.assertIs(obj.__thisclass__, orig.__thisclass__)
+        self.assertIsNot(obj.__self__, p)
+        self.assertIs(type(obj.__self__), PlainSubClass)
+        self.assertIs(obj.__self_class__, PlainSubClass)
+
+    def testSuperBoundCls(self):
+        orig = super(PlainClass, PlainSubClass)
+        self.assertIs(orig.__self__, PlainSubClass)
+        self.assertIs(orig.__thisclass__, PlainClass)
+        self.assertIs(orig.__self_class__, PlainSubClass)
+        p = self.dumpWithPreobjects(None, orig)
+        obj = self.pickler.loads(p)[-1]
+        self.assertIsNot(obj, orig)
+        self.assertIsInstance(obj, type(orig))
+        self.assertIs(obj.__thisclass__, orig.__thisclass__)
+        self.assertIs(obj.__self__, PlainSubClass)
+        self.assertIs(obj.__self_class__, PlainSubClass)
+
+
     # Tests for pickling classes
     
     def testClassicClass(self):
@@ -928,7 +1337,13 @@ class PickelingTest(TestCase):
         self.assertTupleEqual(nt, (1,2))
         self.assertEqual(nt.a, 1)
         self.assertEqual(nt.b, 2)
-        
+
+    def testClassWithCustomDict(self):
+        class C(object):
+            __dict__ = 4711
+        cls = self.classCopyTest(C, dis=False)
+        self.assertEquals(cls().__dict__, 4711)
+
     def testExceptionClass(self):
         class TestException(BaseException):
             attribute = "attribute value"
@@ -980,9 +1395,10 @@ class PickelingTest(TestCase):
         orig = wf_module
         p = self.pickler.dumps(orig)
         importList = self.pickler.getImportList(p)
+        self.assertIsInstance(importList, list)
         for module in importList:
-            print module
-        obj = self.pickler.loads(p)
+            self.assertIsInstance(module, str)
+            self.assertEqual(len(module.split(" ")), 2)
 
     # Handling of resource objects (files, socket, socketpair)
 
@@ -1000,7 +1416,11 @@ class PickelingTest(TestCase):
             
         try:
             orig = (openFile, closedFile, sys.__stdout__, socket_, sp )
-            p = self.pickler.dumps(orig)
+            logging.disable(logging.WARNING)
+            try:
+                p = self.pickler.dumps(orig)
+            finally:
+                logging.disable(0)
             restored = self.pickler.loads(p)
             
             self.assertIsInstance(restored, tuple)
@@ -1059,6 +1479,19 @@ class SPickleToolsTest(TestCase):
         self.assertIs(pt.module_for_globals(pt.module_for_globals), _sPickle)
         self.assertIs(pt.module_for_globals(pt.module_for_globals, withDefiningModules=True), _sPickle)
 
+    def testReducer(self):
+        rvOrig = (1,2,3,4,5)
+        reducer = _sPickle.SPickleTools.reducer(*rvOrig)
+        self.assertIsInstance(reducer, object)
+        rv = reducer.__reduce__()
+        self.assertTupleEqual(rv, rvOrig)
+
+        pickler = _sPickle.SPickleTools()
+        p = pickler.dumps(_sPickle.SPickleTools.reducer(operator.add, (1,2)))
+        obj = pickler.loads(p)
+        self.assertIsInstance(obj, int)
+        self.assertEqual(obj, 3)
+
 if __name__ == "__main__":
     import unittest
     unittest.main()
